#version 450 core

out vec4 gColor;
out vec4 gDepth;

//layout (location = 0) out vec4 FragColor;  
//out vec4 gNormal_reflectionMask;
//out vec4 gAlbedoSpec;
//out vec4 gPosition;

in vec3 Normal;  
in vec3 FragPos; 
in vec2 TexCoords;


#define NR_POINT_LIGHTS 18
#define NR_DIRECT_LIGHTS 5

uniform sampler2D diffuse;
uniform sampler2D specular;
uniform sampler2D ssao;
uniform sampler2D shadowMap;

uniform int numpointlights;
uniform int numdirectlights;
uniform vec3 lightPos[NR_POINT_LIGHTS]; 
uniform vec3 viewPos; 
uniform vec3 lightColor[NR_POINT_LIGHTS];

uniform float constant[NR_POINT_LIGHTS];
uniform float linear[NR_POINT_LIGHTS];
uniform float quadratic[NR_POINT_LIGHTS];

uniform vec3 lightDirect[NR_DIRECT_LIGHTS];
uniform vec3 directLightColor[NR_DIRECT_LIGHTS];
uniform float ambientStrength;
uniform float diffuseStrength;
uniform float specularStrength;
uniform float alpha;
uniform bool gammaEnabled;
uniform float gamma;
uniform bool ambientOcclusionEnabled;

uniform bool isReflective;

float near = 0.1; 
float far  = 200.0; 

float LinearizeDepth(float depth) 
{
    float z = depth * 2.0 - 1.0; // back to NDC 
    return (2.0 * near * far) / (far + near - z * (far - near));	
}


void main()
{
 
vec3 result = vec3(0,0,0);
float AmbientOcclusion = texture(ssao,vec2(gl_FragCoord.x/1920.0,gl_FragCoord.y/1080.0)).r;
float shadow = texture(shadowMap,vec2(gl_FragCoord.x/1920.0,gl_FragCoord.y/1080.0)).r;
for(int i = 0; i < numpointlights; i++) {
	//---------pointlight-------------
    // ambient
	vec3 ambient= vec3(0,0,0);
	if(ambientOcclusionEnabled){
		ambient = ambientStrength * texture(diffuse, TexCoords).rgb * lightColor[i] * AmbientOcclusion;
	} else {
		ambient = ambientStrength * texture(diffuse, TexCoords).rgb * lightColor[i];
	}
    
  	
    // diffuse 
    vec3 norm = normalize(Normal);
    vec3 lightDir = normalize(lightPos[i] - FragPos);
    float diff = max(dot(norm, lightDir), 0.0);
    vec3 diffuse = diffuseStrength * diff * texture(diffuse, TexCoords).rgb * lightColor[i];
    
    // specular
    vec3 viewDir = normalize(viewPos - FragPos);
    vec3 reflectDir = reflect(-lightDir, norm);  
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), alpha);
    vec3 specular = specularStrength * spec * texture(specular, TexCoords).rgb * lightColor[i];  
        
	float distance = length(lightPos[i] - FragPos);
    float attenuation = 1.0 / (constant[i] + linear[i] * distance + quadratic[i] * (distance * distance));    
	
	ambient *= attenuation;
	diffuse *= attenuation;
	specular *= attenuation;
	
    result += (ambient + (diffuse + specular));
	}
	//-------------------------------

	
	for(int i = 0; i < numdirectlights; i++) {
	//----------directLight---------------
	 // ambient
	vec3 directambient= vec3(0,0,0);
	if(ambientOcclusionEnabled){
		directambient = ambientStrength * texture(diffuse, TexCoords).rgb * directLightColor[i] * AmbientOcclusion;
	} else {
		directambient = ambientStrength * texture(diffuse, TexCoords).rgb * directLightColor[i];
	}

    // diffuse 
    vec3 directnorm = normalize(Normal);
    vec3 directlightDir = normalize(-lightDirect[i]);  
    float directdiff = max(dot(directnorm, directlightDir), 0.0);
    vec3 directdiffuse = diffuseStrength * directdiff * texture(diffuse, TexCoords).rgb * directLightColor[i];  
    
    // specular
    vec3 directviewDir = normalize(viewPos - FragPos);
    vec3 directreflectDir = reflect(-directlightDir, directnorm);  
    float directspec = pow(max(dot(directviewDir, directreflectDir), 0.0), alpha);
    vec3 directspecular = specularStrength * directspec * texture(specular, TexCoords).rgb * directLightColor[i];  
	
    result += (directambient + (1-shadow) *(directdiffuse + directspecular));
	
	}
    if(gammaEnabled){
        result = pow(result, vec3(1.0/gamma));
	}
	
	if(numdirectlights==0){
		result += ambientStrength * texture(diffuse, TexCoords).rgb * directLightColor[0];
	}
	
	float depth = LinearizeDepth(gl_FragCoord.z) / far;
    gDepth = vec4(vec3(depth), 1.0);
	
	
	gColor = vec4(result, 1.0);
	
	
} 