#version 450 core
layout (location = 0) out vec3 gColor;
layout (location = 1) out vec3 gNormal;
layout (location = 2) out vec3 gDepth;
layout (location = 3) out vec3 gAlbedoSpec;
layout (location = 4) out vec3 gPosition;
layout (location = 5) out vec3 shadow;

in vec2 TexCoords;
in vec3 FragPos;
in vec3 Normal;
in vec4 FragPosLightSpace;

uniform sampler2D diffuse;
uniform sampler2D spec;
uniform sampler2D shadowMap;

uniform vec3 lightPos;
uniform vec3 viewPos;

float ShadowCalculation(vec4 fragPosLightSpace)
{
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    projCoords = projCoords * 0.5 + 0.5;
    float closestDepth = texture(shadowMap, projCoords.xy).r; 

    float currentDepth = projCoords.z;
	
	vec3 normal = normalize(Normal);
    vec3 lightDir = normalize(lightPos - FragPos);
	
	//float bias = max(0.05 * (1.0 - dot(normal, lightDir)), 0.005); 
	float bias = 0.002;
    //float shadow = currentDepth > closestDepth+bias  ? 1.0 : 0.0;
	
	float shadow = 0.0;
	vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
	for(int x = -1; x <= 1; ++x)
	{
		for(int y = -1; y <= 1; ++y)
		{
			float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
			shadow += currentDepth - bias > pcfDepth ? 1.0 : 0.0;        
		}    
	}
	shadow /= 9.0;
	
	if(projCoords.z > 1.0)
        shadow = 0.0;

    return shadow;
}

void main()
{    
	//vec3 projCoords = FragPosLightSpace.xyz / FragPosLightSpace.w;
	//shadow = vec3(texture(shadowMap, projCoords.xy).r);
	
	shadow = vec3(ShadowCalculation(FragPosLightSpace));
    // store the fragment position vector in the first gbuffer texture
    gPosition = FragPos;
    // also store the per-fragment normals into the gbuffer
    gNormal = normalize(Normal);
    // and the diffuse per-fragment color
    gAlbedoSpec.rgb = vec3(0.95);
}