#version 450 core
out vec4 FragColor;
out vec4 gDepth;

in vec3 Normal;  
in vec3 FragPos; 
in vec2 TexCoords;
flat in ivec4 BoneIDs0;
in vec4 Weights0;


#define NR_POINT_LIGHTS 18
#define NR_DIRECT_LIGHTS 5

uniform sampler2D diffuse;
uniform sampler2D specular;
uniform sampler2D ssao;

uniform int numpointlights;
uniform int numdirectlights;
uniform vec3 lightPos[NR_POINT_LIGHTS]; 
uniform vec3 viewPos; 
uniform vec3 lightColor[NR_POINT_LIGHTS];

uniform float constant[NR_POINT_LIGHTS];
uniform float linear[NR_POINT_LIGHTS];
uniform float quadratic[NR_POINT_LIGHTS];

uniform vec3 lightDirect[NR_DIRECT_LIGHTS];
uniform vec3 directLightColor[NR_DIRECT_LIGHTS];
uniform float ambientStrength;
uniform float diffuseStrength;
uniform float specularStrength;
uniform float alpha;
uniform bool damage;
uniform bool gammaEnabled;
uniform float gamma;

float near = 0.1; 
float far  = 200.0; 

float LinearizeDepth(float depth) 
{
    float z = depth * 2.0 - 1.0; // back to NDC 
    return (2.0 * near * far) / (far + near - z * (far - near));	
}

void main()
{
 
vec3 result = vec3(0,0,0);
float AmbientOcclusion = texture(ssao,vec2(gl_FragCoord.x/1920.0,gl_FragCoord.y/1080.0)).r;
for(int i = 0; i < numpointlights; i++) {
	//---------pointlight-------------
    // ambient
    vec3 ambientPoint = ambientStrength * texture(diffuse, TexCoords).rgb * lightColor[i] * AmbientOcclusion;
  	
    // diffuse 
    vec3 norm = normalize(Normal);
    vec3 lightDir = normalize(lightPos[i] - FragPos);
    float diff = max(dot(norm, lightDir), 0.0);
    vec3 diffusePoint = diffuseStrength * diff * texture(diffuse, TexCoords).rgb * lightColor[i];
    
    // specular
    vec3 viewDir = normalize(viewPos - FragPos);
    vec3 reflectDir = reflect(-lightDir, norm);  
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), alpha);
    vec3 specularPoint = specularStrength * spec * texture(specular, TexCoords).rgb * lightColor[i];  
        
	float distance = length(lightPos[i] - FragPos);
    float attenuation = 1.0 / (constant[i] + linear[i] * distance + quadratic[i] * (distance * distance));    
	
	ambientPoint *= attenuation;
	diffusePoint *= attenuation;
	specularPoint *= attenuation;
	
    result += (ambientPoint + diffusePoint + specularPoint);
	//result += (ambient + diffuse);
	}
	//-------------------------------

	
	for(int i = 0; i < numdirectlights; i++) {
	//----------directLight---------------
	 // ambient
    vec3 directambient = ambientStrength * texture(diffuse, TexCoords).rgb * directLightColor[i] * AmbientOcclusion;
  	
    // diffuse 
    vec3 directnorm = normalize(Normal);
    vec3 directlightDir = normalize(-lightDirect[i]);  
    float directdiff = max(dot(directnorm, directlightDir), 0.0);
    vec3 directdiffuse = diffuseStrength * directdiff * texture(diffuse, TexCoords).rgb * directLightColor[i];  
    
    // specular
    vec3 directviewDir = normalize(viewPos - FragPos);
    vec3 directreflectDir = reflect(-directlightDir, directnorm);  
    float directspec = pow(max(dot(directviewDir, directreflectDir), 0.0), alpha);
    vec3 directspecular = specularStrength * directspec * texture(specular, TexCoords).rgb * directLightColor[i];  
	
    result += (directambient + directdiffuse + directspecular);
	//result += (directambient + directdiffuse);
	if(gammaEnabled)
        	result = pow(result, vec3(1.0/gamma));
    	FragColor = vec4(result , 1.0);
	}
	
	bool found = false;
	
	if(!found){
		if(damage){
			FragColor = vec4(1.0,0.0,0.0, 1.0);
		}else{
    			FragColor = vec4(result , 1.0);
		}
	}
	
	float depth = LinearizeDepth(gl_FragCoord.z) / far;
    gDepth = vec4(vec3(depth), 1.0);
	
} 